unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  dgcspts, dgcslib, DGCSnd, DGCILib, DGC, ExtCtrls, sprites;

type
  TForm1 = class(TForm)
    DGCScreen1: TDGCScreen;
    DGCImageLib1: TDGCImageLib;
    DGCAudio1: TDGCAudio;
    DGCSoundLib1: TDGCSoundLib;
    SpriteEngine1: TSpriteEngine;
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
    procedure DGCScreen1Flip(Sender: TObject);
    procedure DGCScreen1Initialize(Sender: TObject);
    procedure SpriteEngine1Collision(Sender: TSpriteEngine; s1,
      s2: TSprite);
    procedure FormClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    level     : integer;
    score     : integer;
    bombs     : integer;
    gathered  : integer;
    countDown : integer;

    procedure KillBadGuys(flag : boolean);
  public
    { Public declarations }
    procedure NewGame;
    procedure CreateLevel;
    procedure Death;
  end;

var
  Form1 : TForm1;
  ship  : TFractSprite;
  gate  : TSprite;

implementation
uses trace,spriteCode;

type
  TLevel = record
    crystals    : byte;
    mines       : byte;
    bombs       : byte;
    aliens      : integer;
  end;

const
  MaxLevel = 10;
  Sensitivity : real = 0.01;
  Levels : array[1..MaxLevel] of TLevel = (
    (Crystals : 010; mines : 000; bombs : 0; aliens : $000000002),
    (Crystals : 015; mines : 005; bombs : 1; aliens : $000000001),
    (Crystals : 020; mines : 005; bombs : 0; aliens : $000000002),
    (Crystals : 025; mines : 010; bombs : 2; aliens : $000000003),
    (Crystals : 025; mines : 010; bombs : 0; aliens : $000000004),
    (Crystals : 025; mines : 010; bombs : 0; aliens : $000000007),
    (Crystals : 025; mines : 015; bombs : 1; aliens : $000000007),
    (Crystals : 025; mines : 010; bombs : 0; aliens : $000000008),
    (Crystals : 025; mines : 010; bombs : 3; aliens : $00000000F),
    (Crystals : 030; mines : 010; bombs : 0; aliens : $00000000F));


{$R *.DFM}

procedure TForm1.Timer1Timer(Sender: TObject);
var
  p : TPoint;
  nvx,nvy : single;

begin
  if countDown>0 then
    dec(countDown);

  if countDown>0 then
    begin
      SetCursorPos(320,240);
      exit;
    end;

  if not Ship.Visible then
    ship.Show;
    
  GetCursorPos(p);

  nvx:=ship.rvx+((p.x-320)*Sensitivity);
  nvy:=ship.rvy+((p.y-240)*Sensitivity);

//  if ((nvx*nvx)+(nvy*nvy))<64.0 then
    begin
      ship.rvx:=nvx;
      ship.rvy:=nvy;
    end;

{  if ship.rvx<-8 then
    ship.rvx:=-8;
  if ship.rvx>8 then
    ship.rvx:=8;
  if ship.rvy<-8 then
    ship.rvy:=-8;
  if ship.rvy>8 then
    ship.rvy:=8;}

  SetCursorPos(320,240);
end;

procedure TForm1.DGCScreen1Flip(Sender: TObject);
var
  t : integer;

begin
  SpriteEngine1.UpdateSprites;

  if (Ship.x>=Gate.x+8) and (ship.x<=Gate.x+40) and (ship.y>gate.y+8) then
    begin
      inc(level);
      KillBadGuys(TRUE);
      CreateLevel;
    end;

  if random(100)>90 then
    begin
      t:=random(32);
      if (levels[level].aliens and (1 shl t))<>0 then
        case t of
          0 : TGalax.Create;
          1 : TScrager.Create;
        end;
    end;

end;

procedure TForm1.DGCScreen1Initialize(Sender: TObject);
begin
  Ship:=TFractSprite.Create(SpriteEngine1);
  Gate:=TSprite.Create(SpriteEngine1);

  NewGame;
end;

procedure TForm1.SpriteEngine1Collision(Sender: TSpriteEngine; s1,
  s2: TSprite);
begin
  case s1.image of
    0 : // Ship hit something
        case s2.image of
          1 : begin   // Hit a crystal
                s2.Kill;
                inc(gathered);

                if gathered=levels[level].crystals then
                  gate.image:=46;
              end;
           42 : ;
           44 : begin
                  s2.Kill;
                  inc(bombs);
                end;
           45 : begin
                  s2.Kill;
                  death;
                end;
         else
           death;
         end;

    42 : // good Bullet hit something
         case s2.image of
           2..41 : begin  // A bad guy
                     s1.kill;
                     s2.kill;
                   end;
         end;
  end;
end;

procedure TForm1.NewGame;
begin
  Score:=0;
  Bombs:=5;
  Level:=1;
  CreateLevel;
end;

procedure TForm1.CreateLevel;
var
  i : integer;

begin
  Gate.x:=296;
  Gate.y:=446;
  Gate.Image:=47;
  Gate.Show;

  CountDown:=10;
  Ship.rx:=320;
  Ship.ry:=240;
  Ship.rvx:=0;
  Ship.rvy:=0;
  Ship.Image:=0;
  Ship.flags:=Ship.flags or wantStick;
//  Ship.Show;
  Ship.Hide;

  Randomize;

  Gathered:=0;

//  KillBadGuys(TRUE);

  for i:=1 to Levels[level].Crystals do
    with TSprite.Create(SpriteEngine1) do
      begin
        x:=random(SpriteEngine1.Screen.ScreenWidth-64)+32;
        y:=random(SpriteEngine1.Screen.ScreenHeight-64)+32;
        image:=1;
        Show;
      end;

  for i:=1 to Levels[level].Mines do
    with TSprite.Create(SpriteEngine1) do
      begin
        x:=random(SpriteEngine1.Screen.ScreenWidth-64)+32;
        y:=random(SpriteEngine1.Screen.ScreenHeight-64)+32;
        image:=45;
        Show;
      end;

  for i:=1 to Levels[level].bombs do
    with TSprite.Create(SpriteEngine1) do
      begin
        x:=random(SpriteEngine1.Screen.ScreenWidth-64)+32;
        y:=random(SpriteEngine1.Screen.ScreenHeight-64)+32;
        image:=44;
        Show;
      end;
end;

procedure TForm1.Death;
begin
  Ship.rx:=320;
  Ship.ry:=240;
  Ship.rvx:=0;
  Ship.rvy:=0;
  Ship.Image:=0;
  Ship.flags:=Ship.flags or wantStick;
//  Ship.Show;
  Ship.Hide;
  countDown:=10;
  KillBadGuys(FALSE);
end;

procedure TForm1.KillBadGuys(flag : boolean);
var
  temp : TSprite;

begin
  temp:=SpriteEngine1.Sprites.next;
  while temp<>nil do
    begin
      case temp.image of
        0,46,47 : ;
        1,44,45 : if flag then temp.kill;
      else
        temp.kill;
      end;
      temp:=temp.next;
    end;
end;

procedure TForm1.FormClick(Sender: TObject);
begin
  TGoodBullet.Create;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  TraceString('Begining CQ debug session at '+DateTimeToStr(now));
end;

end.
